/**************************************/
/* エネがえる V4 WEBサンプル用          */
/*   © 2020 KOKUSAI KOGYO CO., LTD.   */
/**************************************/

'use strict'

// APIキー、ユーザー名・パスワードが指定されている場合は下記の値を変更してください。
var api_key  = 'UXgDecd1SK3E5cNgbxqcG3D0cQyoDvOC5pqzH6eA';  // APIキー
var username = 'apitest01';                                 // ユーザー名
var password = 'Apitest01';                                 // パスワード

// 固定値
const api_host = 'https://api.enegaeru.com';    // APIホスト名
const prefectures = {
    '01':'北海道','02':'青森県','03':'岩手県','04':'宮城県','05':'秋田県','06':'山形県','07':'福島県','08':'茨城県','09':'栃木県','10':'群馬県',
    '11':'埼玉県','12':'千葉県','13':'東京都','14':'神奈川県','15':'新潟県','16':'富山県','17':'石川県','18':'福井県','19':'山梨県','20':'長野県',
    '21':'岐阜県','22':'静岡県','23':'愛知県','24':'三重県','25':'滋賀県','26':'京都府','27':'大阪府','28':'兵庫県','29':'奈良県','30':'和歌山県',
    '31':'鳥取県','32':'島根県','33':'岡山県','34':'広島県','35':'山口県','36':'徳島県','37':'香川県','38':'愛媛県','39':'高知県','40':'福岡県',
    '41':'佐賀県','42':'長崎県','43':'熊本県','44':'大分県','45':'宮崎県','46':'鹿児島県','47':'沖縄県'
};  // 都道府県コード
const templates = {
    '1': '朝型',
    '2': '昼型',
    '3': '夜型',
    '4': 'オール電化型',
    '-1': 'カスタム（時間帯別入力）'
};  // テンプレートID（生活パターン）

// グローバル変数
var uid = '';              // 認証トークン（API(login) の実行結果によりセット）

// 初期設定
$(async function() {
    // Step0: ログイン　入力項目のセット
    $('#api_key').val(api_key);
    $('#username').val(username);
    $('#password').val(password);

    // Step1: 電気事業者の選択　入力項目（都道府県コード）セット
    $('#prefectures_cd').empty();
    var keys = Object.keys(prefectures);
    keys.sort();
    for(var key of keys) {
        $('#prefectures_cd').append('<option value="' + key + '">' + prefectures[key] + '</option>');
    }

    // Step4: 電気使用量の計算　入力項目（生活パターン）セット
    $('#template_id').empty();
    var keys = Object.keys(templates);
    for(var key of keys) {
        $('#template_id').append('<option value="' + key + '">' + templates[key] + '</option>');
    }
    $('#template_id').val(3);   // デフォルトを夜型にする
    $('#customInput').hide();

    // タブのナビゲーション　クリック時の動作設定
    $('.tabs_nav div').click( function() {
        if ($(this).hasClass('enable')) {
            var tabname = $(this).attr('id').replace('nav_', '');
            activate(tabname);
        }
    });

    // 「次へ」　クリック時の動作設定
    $('.next_button').click( function() {
        var tabname = $(this).parent().attr('id').replace('tab_', '');
        next_click(tabname);
    })

    // タブの初期表示
    $('.tabs_body > div').hide();
    $('#tab_top').show();

    // 「次へ」ボタンの初期表示
    $('.next_button').prop('disabled', true);
    $('#tab_top .next_button').prop('disabled', false);

});

// API(login) 実行ボタンが押された時の動作
async function login_click() {

    // APIパラメータのセット
    var data = {
        'id': $('#username').val(),
        'password': $('#password').val(),
        'forcelogin': true
    };

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/login',
        method: 'POST',
        data: data,
    });
    $('#loading').hide();
    console.log('login res:' + JSON.stringify(res));

    var msg = '<h3>セットしたパラメータ</h3>' + JSON.stringify(data);
    msg += '<h3>実行結果</h3>';

    if (res.error) {
        msg += '<p>エラー：' + JSON.stringify(res) + '</p>';
        uid = '';
    } else {
        uid = res.uid;  // 認証トークンをグローバル変数にセット
        msg += '<p>成功：</p><p>' + JSON.stringify(res) + '</p>';
        msg += 'これ以降はこの認証トークン(uid)をヘッダにセットして APIをコールします。</p>';

        $('#tab_login .next_button').prop('disabled', false);
    }
    $('#login_msg').html(msg);
}

// API 呼び出し部分
async function requestAjax(req) {

    var data = (req.method == 'GET')? req.data : JSON.stringify(req.data);

    var headers = {
        'x-api-key': api_key,   // APIキーのセット
        'Content-Type': 'application/json'
    };
    if (req.apiname != '/v4/login') headers.Authorization = uid;    // 認証トークンのセット

    var options = {
        'url': api_host + req.apiname,
        'method': req.method,
        'dataType': 'json',
        'contentType': 'application/json; charset=UTF-8',
        'headers': headers,
        'data': data
    };

    var res = await $.ajax(options)
        .then((...args) => {
            const [data, textStatus, jqXHR] = args;
            return data;
        })
        .catch((...args) => {
            const [jqXHR, textStatus, errorThrown] = args;

            swal('', 'ステータスコード：'+ jqXHR.status, 'error');
                // このWEBサンプルでは エラー発生時に ステータスコードをアラート表示して、内容は 各タブの '実行結果' に表示する仕様としています
            switch (jqXHR.status) {
                case 400:
                case 500:
                    return jqXHR.responseJSON;
                case 504:
                    return {'error':{'message':'タイムアウトしました。再実行をお願いします。'}};
                default:
                    return {'error':{'message':'予期しないエラーが発生しました。'}};
            }
        });
    return res;
}

// タブの切り替え
function activate(tabname) {
    $('.tabs_nav div').removeClass('active');
    $('#nav_'+tabname).addClass('active');
    $('#nav_'+tabname).addClass('enable');
    $('.tabs_body > div').hide();
    if (($('#div_chart').length)
        && (['top', 'login', 'bottom'].indexOf(tabname) == -1)) {
        $('.tabs_body').css({ 'width': '45%' });
        $('#div_chart').show();
    } else {
        $('.tabs_body').css({ 'width': '82%' });
        $('#div_chart').hide();
    }
    $('#tab_'+tabname).show();
    if ($('#div_chart').length) drawChart(tabname);
    $('body,html').animate({ scrollTop: 0 }, 500);
}

// 「次へ」ボタンが押された時の動作
function next_click(tabname) {
    activate(tabs[tabs.indexOf(tabname)+1]);
}

// activeな tabname を取得
function getActiveTabName() {
    var obj = $('.tabs_nav .active');
    var tabname = obj.attr('id').replace('nav_', '')
    return tabname;
}
