/**************************************/
/* エネがえる V4 WEBサンプル用          */
/*   © 2020 KOKUSAI KOGYO CO., LTD.   */
/**************************************/

'use strict'

// 固定値
const tabs = ['top','login','epcorps','epplans','selectepplan','usepowercalc','epchargecalc','extractepplans','epdiagnosis','bottom'];// このWEBアプリで使用しているタブ

// グローバル変数
var epcorps  = null;            // API(epcorps) の実行結果
var epplans  = null;            // API(epplans) の実行結果
var usepower = null;            // API(usepowercalc) の実行結果
var extractepplans = null;      // API(extractepplans) の実行結果

// API(epcorps) 実行ボタンが押された時の動作
async function epcorps_click() {

    // APIパラメータのセット
    var data = {
        'prefectures_cd': $('#prefectures_cd').val()
    };

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/epcorps',
        method: 'GET',
        data: data
    });
    $('#loading').hide();
    console.log('epcorps res:' + JSON.stringify(res));

    var msg = '<h3>セットしたパラメータ</h3>' + JSON.stringify(data);
    msg += '<h3>実行結果</h3>';

    $('#epcorp_cd').empty();
    if (res.error) {
        msg += '<p>エラー：' + JSON.stringify(res) + '</p>';
        epcorps = null;
    } else {
        epcorps = res;  // 電気事業者のリストをグローバル変数にセット
        msg += '<p>成功！ 電気事業者の選択肢を Step2 にセットしました。- ' + epcorps.length + ' 事業者</p>';
        msg += '<p>（先頭 10件の事業者名のみ表示）<br/>';
        epcorps.forEach((elem, idx) => {
            $('#epcorp_cd').append('<option value="' + elem.id + '">' + elem.epcorp_short_name + '</option>');
            if (idx < 10) {
                msg += ' - ' + elem.epcorp_short_name + '<br/>';
            } else if (idx == 10) {
                msg += '（以下 省略）</p>'
            }
        });

        $('#tab_epcorps .next_button').prop('disabled', false);
    }
    $('#epcorps_msg').html(msg);
}

// API(epplans) 実行ボタンが押された時の動作
async function epplans_click() {

    // APIパラメータのセット
    var data =  {
        'epcorp_cd' : $('#epcorp_cd').val()
    }

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/epplans',
        method: 'GET',
        data: data
    });
    $('#loading').hide();
    console.log('epplans res:' + JSON.stringify(res));

    var msg = '<h3>セットしたパラメータ</h3>' + JSON.stringify(data);
    msg += '<h3>実行結果</h3>';

    $('#epplan_cd').empty();
    if (res.error) {
        msg += '<p>エラー：' + JSON.stringify(res) + '</p>';
        epplans = null;
    } else {
        epplans = res;  // 料金プランのリストをグローバル変数にセット
        msg += '<p>成功！ 料金プランの選択肢を Step3 にセットしました。- ' + epplans.length + ' プラン</p>';
        msg += '<p>（先頭 10件のプラン名のみ表示）<br/>';
        epplans.forEach((elem, idx) => {
            $('#epplan_cd').append('<option value="' + elem.plan_cd + '">' + elem.plan_name + '</option>');
            if (idx < 10) {
                msg += '-' + elem.plan_name + '<br/>';
            } else if (idx == 10) {
                msg += '（以下 省略）</p>'
            }
        });

        $('#tab_epplans .next_button').prop('disabled', false);
    }
    $('#epplans_msg').html(msg);
}

// 「料金プランの選択」ボタンが押された時の動作
function selectepplan_click() {
    var epplan = getEpplan();
    delete epplan.requirement;  // v3用のデータ かつ JSON形式のため、除外

    var msg = '<h3>選択した料金プラン</h3>' + JSON.stringify(epplan);   // 内容が短いため、そのまま表示
    $('#selectepplan_msg').html(msg);

    $('#tab_selectepplan .next_button').prop('disabled', false);
}

// 生活スタイルがカスタムかどうかで時間帯別入力の表示切り替え
function templateChanged() {
    if ($('#template_id').val() == -1) {
        $('#customInput').show();
    } else {
        $('#customInput').hide();
    }
}

// API(usepowercalc) 実行ボタンが押された時の動作
async function usepowercalc_click() {

    // APIパラメータのセット
    var data = {
        'prefectures_cd': $('#prefectures_cd').val(),
        'template_id'   : $('#template_id').val(),
        'average_epower': $('#average_epower').val(),
        'epowers' : [
            $('#ep1' ).val(),
            $('#ep2' ).val(),
            $('#ep3' ).val(),
            $('#ep4' ).val(),
            $('#ep5' ).val(),
            $('#ep6' ).val(),
            $('#ep7' ).val(),
            $('#ep8' ).val(),
            $('#ep9' ).val(),
            $('#ep10').val(),
            $('#ep11').val(),
            $('#ep12').val()
        ]
    }

    if (data.template_id==-1) {
        // 時間帯別入力を APIパラメータとしてセット
        data.timeperiod = [];
        for (var i=1; i<4; i++) {
            var oneperiod = {
                from  : $('#from'  + i).val(),
                to    : $('#to'    + i).val(),
                ratio : $('#ratio' + i).val()
            }
            data.timeperiod.push(oneperiod);
        }
    } else {
        // WEB画面上の時間帯別入力をクリア
        for (var i=1; i<4; i++) {
            $('#from'  + i).val('');
            $('#to'    + i).val('');
            $('#ratio' + i).val('');
        }
    }

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/usepowercalc',
        method: 'POST',
        data: data
    });
    $('#loading').hide();
    console.log('usepowercalc res:' + JSON.stringify(res));

    var msg = '<h3>セットしたパラメータ</h3>' + JSON.stringify(data);
    msg += '<h3>実行結果</h3>';
    if (res.error) {
        msg += '<p>エラー：' + JSON.stringify(res) + '</p>';
        usepower = null;
    } else {
        msg += '<p>成功！ 計算結果を Step5 にセットしました。</p>';
        usepower = res;  // 電気使用量計算の結果をグローバル変数にセット
        msg += '<p>各月の使用量合計は ' + JSON.stringify(calcMonthlyAmount()) + '（長さ 12の配列）</p>';
        msg += '<p>月毎・時間毎の使用量（平日）は<br/>' + JSON.stringify(usepower.weekday.day_usepower) + '</p>';
        msg += '<p>月毎・時間毎の使用量（休日）は<br/>' + JSON.stringify(usepower.holiday.day_usepower) + '</p>';
        msg += '<p>月毎・時間毎の使用量（平均）は <br/> ' + JSON.stringify(usepower.average.day_usepower) + '</p>';
        msg += '<p>（各々 12 x 24 の配列。現在の生活スタイルテンプレートは 平日・休日に同じものを使用しているため同じ値となっています。）</p>'

        $('#tab_usepowercalc .next_button').prop('disabled', false);
    }
    $('#usepowercalc_msg').html(msg);
}

// 各月使用量合計の計算
// API(usepowercalc) の各月データは1日分のデータのため、月全体の使用量の計算は以下のように行います。
function calcMonthlyAmount() {

    var data = usepower.average.day_usepower;
    const MONTHDAY = [31,28,31,30,31,30,31,31,30,31,30,31]; // うるう年は考慮せず。2月は 28日で固定。

    var ret = [];
    for (var month=0; month<12; month++) {
        var onedaytotal = 0;
        for (var hour=0; hour<24; hour++) {
            onedaytotal += data[month][hour];
        }
        ret.push(Math.round(parseFloat(onedaytotal * MONTHDAY[month]) * 100) / 100);
    }

    return ret;
}

// API(epchargecalc) 実行ボタンが押された時の動作
async function epchargecalc_click() {

    // APIパラメータのセット
    var data = {
        'epcorp_cd': $('#epcorp_cd').val(),
        'plan_cd'  : $('#epplan_cd').val()
    }

    var epplan = getEpplan();
    if (epplan.ampare) {
        data.ampare = epplan.ampare;
    } else {
        data.capacity = epplan.default_vol;
    }

    data.basemonth = usepower.basemonth;
    data.weekday   = {};
    data.weekday.day_purchase = usepower.weekday.day_usepower;
    data.holiday   = {};
    data.holiday.day_purchase = usepower.holiday.day_usepower;

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/epchargecalc',
        method: 'POST',
        data: data
    });
    $('#loading').hide();
    console.log('epchargecalc res:' + JSON.stringify(res));

    data.weekday = '(表示省略)';
    data.holiday = '(表示省略)';
    var msg = '<h3>セットしたパラメータ</h3>' + JSON.stringify(data);
    msg += '<h3>実行結果</h3>' + JSON.stringify(res);   // 内容が短いため、そのまま表示
    if (res.error) {
    } else {
        $('#tab_epchargecalc .next_button').prop('disabled', false);
    }
    $('#epchargecalc_msg').html(msg);
}

// API(extractepplans) 実行ボタンが押された時の動作
async function extractepplans_click() {

    // APIパラメータのセット
    var data = {};

    var epplan = getEpplan();
    data.contract_type = epplan.contract_type;
    if (epplan.ampare) {
        data.ampare = epplan.ampare;
    } else {
        data.capacity = epplan.default_vol;
    }

    data.epcorps = [];
    for (var i=0; i<4; i++) {               // サンプルのため、契約可能な電気事業者の内、先頭の４社のみを選択
        data.epcorps.push(epcorps[i].id);
    }

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/extractepplans',
        method: 'POST',
        data: data
    });
    $('#loading').hide();
    console.log('extractepplans res:' + JSON.stringify(res));

    var msg = '<h3>セットしたパラメータ</h3>' + JSON.stringify(data);
    msg += '<h3>実行結果</h3>';
    msg += '<p>各事業者の該当するプラン</p>';

    if (res.error) {
        msg += JSON.stringify(res);
        extractepplans = null;
    } else {
        extractepplans = res;   // 結果をグローバル変数にセット
        extractepplans.forEach(item => {
            msg += '<p>' + JSON.stringify(item) + '</p>';
        })

        $('#tab_extractepplans .next_button').prop('disabled', false);
    }
    $('#extractepplans_msg').html(msg);
}

// API(epdiagnosis) 実行ボタンが押された時の動作
async function epdiagnosis_click() {

    // APIパラメータのセット
    var data = {};

    var epplan = getEpplan();
    if (epplan.ampare) {
        data.ampare = epplan.ampare;
    } else {
        data.capacity = epplan.default_vol;
    }

    data.calcplans = [];
    extractepplans.forEach(corp => {
        var onecorp = {};
        onecorp.epcorp_cd = corp.epcorp_cd;
        onecorp.plans = [];
        corp.plans.forEach(plan => {
            onecorp.plans.push(plan.plan_cd);
        })
        data.calcplans.push(onecorp);
    });

    data.basemonth = usepower.basemonth;
    data.weekday   = {};
    data.weekday.day_purchase = usepower.weekday.day_usepower;
    data.holiday   = {};
    data.holiday.day_purchase = usepower.holiday.day_usepower;

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/epdiagnosis',
        method: 'POST',
        data: data
    });
    $('#loading').hide();
    console.log('epdiagnosis res:' + JSON.stringify(res));

    data.weekday = '(表示省略)';
    data.holiday = '(表示省略)';
    var msg = '<h3>セットしたパラメータ</h3>' + JSON.stringify(data);
    msg += '<h3>実行結果</h3>';
    msg += '<p>各プランの料金（年額の安価な順）</p>';

    if (res.error) {
        msg += JSON.stringify(res);
    } else {
        res.forEach(item => {
            msg += '<p>' + JSON.stringify(item) + '</p>';
        })
        $('#tab_epdiagnosis .next_button').prop('disabled', false);
    }
    $('#epdiagnosis_msg').html(msg);
}

// 選択した料金プランの情報を保存済みの配列から取り出す
function getEpplan() {
    var epplan_cd = $('#epplan_cd').val();
    for (var i=0; i<epplans.length; i++) {
        if (epplans[i].plan_cd == epplan_cd) {
            return epplans[i];
        }
    }
}
