/**************************************/
/* エネがえる V4 WEBサンプル用          */
/*   © 2020 KOKUSAI KOGYO CO., LTD.   */
/**************************************/

'use strict'

// 固定値
const tabs = ['top','login','usepowercalc','pvpowercalc','pvSimulation','cellSimulation','bottom']; // このWEBアプリで使用しているタブ
const colors = {
    'pv2sell'   : 'rgba(242,218, 25,1)',    // 太陽光 → 売電
    'pv2self'   : 'rgba(252,191, 74,1)',    // 太陽光 → 自家消費
    'pv2self_l' : 'rgba(252,191, 74,0.8)',  // 太陽光 → 自家消費(線)
    'pv2cell'   : 'rgba(152,212, 61,1)',    // 太陽光 → 蓄電池
    'ep2self'   : 'rgba( 82,136,184,1)',    // 系統 → 自家消費
    'ep2cell'   : 'rgba(147,195,240,1)',    // 系統 → 蓄電池
    'cell2self' : 'rgba(152,212, 61,1)',    // 蓄電池 → 自家消費
}   // グラフで使用している配色

// グローバル変数
var usepower   = null;      // API(usepowercalc) の実行結果
var cells      = null;      // API(cells)の実行結果
var pvpower    = null;      // API(pvpowercalc) の実行結果
var h_chartdata = {};       // グラフ描画用データ
var chartMaxVal = 0;        // グラフ縦軸最大値
var dayChart   = null;      // Chartオブジェクト

// 生活スタイルがカスタムかどうかで時間帯別入力の表示切り替え
function templateChanged() {
    if ($('#template_id').val() == -1) {
        $('#customInput').show();
    } else {
        $('#customInput').hide();
    }
}

// API(usepowercalc) 実行ボタンが押された時の動作
async function usepowercalc_click() {

    // APIパラメータのセット
    var data = {
        'prefectures_cd': $('#prefectures_cd').val(),
        'template_id'   : $('#template_id').val(),
        'average_epower': $('#average_epower').val(),
        'epowers' : [
            $('#ep1' ).val(),
            $('#ep2' ).val(),
            $('#ep3' ).val(),
            $('#ep4' ).val(),
            $('#ep5' ).val(),
            $('#ep6' ).val(),
            $('#ep7' ).val(),
            $('#ep8' ).val(),
            $('#ep9' ).val(),
            $('#ep10').val(),
            $('#ep11').val(),
            $('#ep12').val()
        ]
    }

    if (data.template_id==-1) {
        // 時間帯別入力を APIパラメータとしてセット
        data.timeperiod = [];
        for (var i=1; i<4; i++) {
            var oneperiod = {
                from  : $('#from'  + i).val(),
                to    : $('#to'    + i).val(),
                ratio : $('#ratio' + i).val()
            }
            data.timeperiod.push(oneperiod);
        }
    } else {
        // WEB画面上の時間帯別入力をクリア
        for (var i=1; i<4; i++) {
            $('#from'  + i).val('');
            $('#to'    + i).val('');
            $('#ratio' + i).val('');
        }
    }

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/usepowercalc',
        method: 'POST',
        data: data
    });
    $('#loading').hide();
    console.log('usepowercalc res:' + JSON.stringify(res));

    var msg = '<h3>実行結果</h3>';
    if (res.error) {
        msg += JSON.stringify(res);
        usepower = null;

        setChartData('usepowercalc', null);
        drawChart();

    } else {
        msg += '<p>成功！ 計算結果をチャートで表示します。</p>';
        usepower = res;  // 電気使用量計算の結果をグローバル変数にセット

        setChartData('usepowercalc', {'day_ep2self':usepower.average.day_usepower});
        drawChart();
        $('#usepowercalc_msg').html(msg);

        await setSunpoints(data.prefectures_cd);    // Step2:太陽光発電量計算の 日射量観測地点の選択肢のセット
        msg += '<p>日射量観測地点を Step2の選択肢に追加しました。</p>'
        $('#usepowercalc_msg').html(msg);

        await setCells();                           // Step4:太陽光・蓄電池利用シミュレーションの 蓄電池の選択肢のセット
        msg += '<p>蓄電池を Step4の選択肢に追加しました。</p>'
        $('#usepowercalc_msg').html(msg);

        $('#tab_usepowercalc .next_button').prop('disabled', false);
    }

}

// Step2:太陽光発電量計算の 日射量観測地点の選択肢のセット (Step1の成功時に呼ばれる)
async function setSunpoints(prefectures_cd) {

    // APIパラメータのセット
    var data = {
        'prefectures_cd': prefectures_cd
    }

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/sunpoints',
        method: 'GET',
        data: data
    });
    $('#loading').hide();
    console.log('sunpoints res:' + JSON.stringify(res));

    $('#point_no').empty();
    if (res.error) {
        swal ('', '予期せぬエラーが発生しました', 'error');
    } else {
        var sunpoints  = res;
        var pref_capital_no = 0;

        sunpoints.forEach(elem => {
            if (elem.pref_capital) pref_capital_no = elem.point_no;
            $('#point_no').append('<option value="' + elem.point_no + '">' + elem.point_name + '</option>');
        });
        $('#point_no').val(pref_capital_no);
    }
}

// Step4:蓄電池利用シミュレーションの 蓄電池の選択肢のセット (Step1の成功時に呼ばれる)
async function setCells() {

    // APIパラメータのセット
    var data = {};

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/cells',
        method: 'GET',
        data: data
    });
    $('#loading').hide();
    console.log('cells res:' + JSON.stringify(res));

    $('#cell_id').empty();
    if (res.error) {
        swal ('', '予期せぬエラーが発生しました', 'error');
        cells = null;
    } else {
        cells = res;  // 蓄電池のリストをグローバル変数にセット

        cells.forEach(elem => {
            $('#cell_id').append('<option value="' + elem.id + '">' + elem.name + '</option>');
        });
        $('#cell_id').val(cells[0].id);
        setCellDefault();
    }
}

// API(pvpowercalc) 実行ボタンが押された時の動作
async function pvpowercalc_click() {

    // APIパラメータのセット
    var data = {
        'point_no'         : $('#point_no').val(),
        'pcs_output'       : $('#pcs_output').val(),
        'pcs_conversion'   : $('#pcs_conversion').val(),
        'panels': [
            {
                'vol': $('#vol1').val(),
                'azimuth': $('#azimuth1').val(),
                'tilt': $('#tilt1').val(),
                'installation': 2,
                'basic_coeff': 0.76,
                'maxtemp_coeff': -0.44
            }
        ]

    }

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/pvpowercalc',
        method: 'POST',
        data: data
    });
    $('#loading').hide();
    console.log('pvpowercalc res:' + JSON.stringify(res));

    var msg = '<h3>実行結果</h3>';
    if (res.error) {
        msg += JSON.stringify(res);
        pvpower = null;

        setChartData('pvpowercalc', null);
        drawChart();

    } else {
        msg += '<p>成功！ 計算結果をチャートで表示します。<br/>（チャートでは 簡略化のため、PCS出力を「自家消費」、過積載分を「売電量」で表示しています。）</p>';
        pvpower = res;  // 太陽光発電量計算の結果をグローバル変数にセット

        setChartData(
            'pvpowercalc',
            {
                'day_pv2self':pvpower.pcs_output,       // PCS出力を「自家消費」として表示（本来は違うデータであることに注意）
                'day_pv2sell':pvpower.pcs_roundoff      // 過積載分を「売電量」として表示（本来は違うデータであることに注意）
            }
        );
        drawChart();

        $('#tab_pvpowercalc .next_button').prop('disabled', false);
    }
    $('#pvpowercalc_msg').html(msg);

}

//　Step3:太陽光発電利用シミュレーションで API(pvcellsimulation) 実行ボタンが押された時の動作
async function pvSimulation_click() {

    // APIパラメータのセット
    var data = {
        'day_usepower'     : usepower.average.day_usepower,
        'pvpowers'         : pvpower,
        'selling_mode'     : 1  // 1:余剰売電, 2:全量売電
    }

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/pvcellsimulation',
        method: 'POST',
        data: data
    });
    $('#loading').hide();
    console.log('pvcellsimulation res:' + JSON.stringify(res));

    var msg = '<h3>実行結果</h3>';
    if (res.error) {
        msg += JSON.stringify(res);

        setChartData('pvSimulation', null);
        drawChart();

    } else {
        msg += '<p>成功！ 計算結果をチャートで表示します。</p>';

        setChartData('pvSimulation', res);
        drawChart();

        $('#tab_pvSimulation .next_button').prop('disabled', false);
    }
    $('#pvSimulation_msg').html(msg);

}

//　Step4:太陽光発電・蓄電池利用シミュレーションで充電の優先順位または蓄電池が変更された時の動作
function setCellDefault() {
    var cell_id = $('#cell_id').val();
    var cell_info = cells.filter(item => item.id == cell_id)[0];

    var charge_priority = $('#charge_priority').val();

    $('#actual_capacity').val(cell_info.charge_depth - cell_info.discharge_depth);
    $('#twocycle').prop('checked', (cell_info.twocycle_default == 1) ? true : false);

    if (charge_priority == 2) {
        $('#charge_from').val(cell_info.charge_from);
        $('#charge_to').val(cell_info.charge_to);
        $('#discharge_from').val(cell_info.discharge_from);
        $('#discharge_to').val(cell_info.discharge_to);
    } else {
        $('#charge_from').val(cell_info.charge_from_a);
        $('#charge_to').val(cell_info.charge_to_a);
        $('#discharge_from').val(cell_info.discharge_from_a);
        $('#discharge_to').val(cell_info.discharge_to_a);
    }
}

//　Step4:太陽光発電・蓄電池利用シミュレーションで API(pvcellsimulation) 実行ボタンが押された時の動作
async function cellSimulation_click() {

    // APIパラメータのセット（蓄電池の詳細設定をユーザー入力に置き換えることも可能ですが、今回は選択した蓄電池の規定値をそのままセットしています）
    var cell_id   = $('#cell_id').val();

    var data = {
        'day_usepower' : usepower.average.day_usepower,
        'pvpowers'     : pvpower,
        'selling_mode' : 1,
        'charge_priority' : $('#charge_priority').val(),
        'cell_info'    : {
            'id'              : cell_id,
            'actual_capacity' : $('#actual_capacity').val(),
            'charge_from'     : $('#charge_from').val(),
            'charge_to'       : $('#charge_to').val(),
            'discharge_from'  : $('#discharge_from').val(),
            'discharge_to'    : $('#discharge_to').val(),
            'twocycle'        : ($('#twocycle').prop('checked')? 1 : 0)
        }
    }

    // APIの呼び出し
    $('#loading').show();
    var res = await requestAjax( {
        apiname: '/v4/pvcellsimulation',
        method: 'POST',
        data: data
    });
    $('#loading').hide();
    console.log('pvcellsimulation res:' + JSON.stringify(res));

    var msg = '<h3>実行結果</h3>';
    if (res.error) {
        msg += JSON.stringify(res);

        setChartData('cellSimulation', null);
        drawChart();

    } else {
        msg += '<p>成功！ 計算結果をチャートで表示します。</p>';

        setChartData('cellSimulation', res);
        drawChart();

        $('#tab_cellSimulation .next_button').prop('disabled', false);
    }
    $('#cellSimulation_msg').html(msg);

}

// API実行結果をグラフ描画用データに保存
function setChartData(tabname, data) {

    if (!data) {
        h_chartdata[tabname] = null;
        return;
    }

    // グラフ描画用データ保存
    var chartdata = {
        'day_ep2self'   : (data.day_ep2self  )? data.day_ep2self   : (new Array(12)).fill( (new Array(24)).fill(0) ),
        'day_ep2cell'   : (data.day_ep2cell  )? data.day_ep2cell   : (new Array(12)).fill( (new Array(24)).fill(0) ),
        'day_pv2cell'   : (data.day_pv2cell  )? data.day_pv2cell   : (new Array(12)).fill( (new Array(24)).fill(0) ),
        'day_pv2self'   : (data.day_pv2self  )? data.day_pv2self   : (new Array(12)).fill( (new Array(24)).fill(0) ),
        'day_pv2sell'   : (data.day_pv2sell  )? data.day_pv2sell   : (new Array(12)).fill( (new Array(24)).fill(0) ),
        'day_cell2self' : (data.day_cell2self)? data.day_cell2self : (new Array(12)).fill( (new Array(24)).fill(0) )
    };

    h_chartdata[tabname] = chartdata;   // タブ別に保存

    // グラフ縦軸の最大値の設定
    for (var month=0; month<12; month++) {
        for (var hour=0; hour<24; hour++) {
            chartMaxVal = Math.max(
                chartMaxVal,
                chartdata.day_ep2self[month][hour] + chartdata.day_ep2cell[month][hour] + chartdata.day_cell2self[month][hour] + chartdata.day_pv2self[month][hour],
                chartdata.day_pv2self[month][hour] + chartdata.day_pv2cell[month][hour] + chartdata.day_pv2sell[month][hour]
            );
        }
    }
    chartMaxVal = Math.ceil(chartMaxVal);

}

// グラフの描画
function drawChart() {

    var chartdata = h_chartdata[getActiveTabName()];

    if (!chartdata) {
        $('#div_chart').hide();
        return;
    } else {
        $('#div_chart').show();
    }

    var month = ($('#selmonth').val())? $('#selmonth').val() : 0;
    if (dayChart) dayChart.destroy();
    dayChart = new Chart($('#dayChart'), {
        type: 'bar',
        data: {
            labels: ['0:00','1:00','2:00.','3:00','4:00','5:00','6:00','7:00','8:00','9:00','10:00','11:00','12:00','13:00','14:00','15:00','16:00','17:00','18:00','19:00','20:00','21:00','22:00','23:00'],
            datasets: [
                {
                    label: '太陽光→自家消費',
                    borderColor: colors['pv2self'],
                    borderWidth:1,
                    backgroundColor: colors['pv2self'],
                    data: chartdata.day_pv2self[month]
                },{
                    label: '系統→自家消費',
                    borderColor: colors['ep2self'],
                    borderWidth:1,
                    backgroundColor: colors['ep2self'],
                    data: chartdata.day_ep2self[month]
                },{
                    label: '蓄電池→自家消費',
                    borderColor: colors['cell2self'],
                    borderWidth:1,
                    backgroundColor: colors['cell2self'],
                    data: chartdata.day_cell2self[month]
                },{
                    label: '系統→蓄電',
                    borderColor: colors['ep2cell'],
                    borderWidth:1,
                    backgroundColor: colors['ep2cell'],
                    data: chartdata.day_ep2cell[month]
                }, {
                    type: 'line',
                    label: '太陽光→自家消費',
                    borderColor: 'rgba(125,125,125,1)',
                    borderWidth: 1,
                    pointBackgroundColor: colors['pv2self'],
                    backgroundColor: colors['pv2self_l'],
                    data: chartdata.day_pv2self[month]
                }, {
                    type: 'line',
                    label: '太陽光→蓄電',
                    borderColor: 'rgba(125,125,125,1)',
                    borderWidth: 1,
                    pointBackgroundColor: colors['pv2cell'],
                    backgroundColor: colors['pv2cell'],
                    data: chartdata.day_pv2cell[month]
                },{
                    type: 'line',
                    label: '太陽光→売電',
                    borderColor: 'rgba(125,125,125,1)',
                    borderWidth: 1,
                    pointBackgroundColor: colors['pv2sell'],
                    backgroundColor: colors['pv2sell'],
                    data: chartdata.day_pv2sell[month]
                }
            ]
        },
        options: {
            scales: {
                xAxes: [{
                      stacked: true,
                      categoryPercentage:0.4
                }],
                yAxes: [{
                      stacked: true,
                      ticks: {
                        min: 0,
                        max: chartMaxVal
                      }
                }]
            },
            legend: {
                labels: {
                    fontSize:10,
                    boxWidth:12,
                    padding:10,
                    generateLabels: function(chart){
                        return  chart.data.datasets.map(function(dataset, i) {
                            return {
                                text: dataset.label,
                                fillStyle: dataset.backgroundColor,
                                hidden: dataset.hidden,
                                lineCap: dataset.borderCapStyle,
                                lineDash: dataset.borderDash,
                                lineDashOffset: dataset.borderDashOffset,
                                lineJoin: dataset.borderJoinStyle,
                                lineWidth: 5,
                                strokeStyle: dataset.borderColor,
                                pointStyle: dataset.pointStyle,
                                datasetIndex: i
                            };
                        });
                    }

                },
                display: false,
                position: 'bottom'
            },
            tooltips:{
              mode:'label'
            }
        }
    });

}
